@@ -17,12 +17,9 @@
 #' @exportMethod nlines
 setMethod(
   "nlines", "TableRow",
-  function(x, colwidths, max_width) {
-    ## XXX this is wrong and needs to be fixed
-    ## should not be hardcoded here
-    col_gap <- 3L
-    fns <- sum(unlist(lapply(row_footnotes(x), nlines, max_width = max_width))) +
-      sum(unlist(lapply(cell_footnotes(x), nlines, max_width = max_width)))
+  function(x, colwidths, max_width, fontspec, col_gap = 3) {
+    fns <- sum(unlist(lapply(row_footnotes(x), nlines, max_width = max_width, fontspec = fontspec))) +
+      sum(unlist(lapply(cell_footnotes(x), nlines, max_width = max_width, fontspec = fontspec)))
     fcells <- as.vector(get_formatted_cells(x))
     spans <- row_cspans(x)
     have_cw <- !is.null(colwidths)
@@ -48,9 +45,18 @@ setMethod(
     ## rowext <- max(vapply(strsplit(c(obj_label(x), fcells), "\n", fixed = TRUE),
     ##                      length,
     ##                      1L))
-    rowext <- max(unlist(mapply(function(s, w) {
-      nlines(strsplit(s, "\n", fixed = TRUE), max_width = w)
-    }, s = c(obj_label(x), fcells), w = (colwidths %||% max_width) %||% 1000L, SIMPLIFY = FALSE)))
+    rowext <- max(
+      unlist(
+        mapply(
+          function(s, w) {
+            nlines(strsplit(s, "\n", fixed = TRUE), max_width = w, fontspec = fontspec)
+          },
+          s = c(obj_label(x), fcells),
+          w = (colwidths %||% max_width) %||% vector("list", length(c(obj_label(x), fcells))),
+          SIMPLIFY = FALSE
+        )
+      )
+    )
 
     rowext + fns
   }
@@ -60,10 +66,10 @@ setMethod(
 #' @rdname formatters_methods
 setMethod(
   "nlines", "LabelRow",
-  function(x, colwidths, max_width) {
+  function(x, colwidths, max_width, fontspec = fontspec, col_gap = NULL) {
     if (labelrow_visible(x)) {
-      nlines(strsplit(obj_label(x), "\n", fixed = TRUE)[[1]], max_width = colwidths[1]) +
-        sum(unlist(lapply(row_footnotes(x), nlines, max_width = max_width)))
+      nlines(strsplit(obj_label(x), "\n", fixed = TRUE)[[1]], max_width = colwidths[1], fontspec = fontspec) +
+        sum(unlist(lapply(row_footnotes(x), nlines, max_width = max_width, fontspec = fontspec)))
     } else {
       0L
     }
@@ -74,8 +80,8 @@ setMethod(
 #' @rdname formatters_methods
 setMethod(
   "nlines", "RefFootnote",
-  function(x, colwidths, max_width) {
-    nlines(format_fnote_note(x), colwidths = colwidths, max_width = max_width)
+  function(x, colwidths, max_width, fontspec, col_gap = NULL) {
+    nlines(format_fnote_note(x), colwidths = colwidths, max_width = max_width, fontspec = fontspec)
   }
 )
 
@@ -83,12 +89,24 @@ setMethod(
 #' @rdname formatters_methods
 setMethod(
   "nlines", "InstantiatedColumnInfo",
-  function(x, colwidths, max_width) {
+  function(x, colwidths, max_width, fontspec, col_gap = 3) {
     h_rows <- .do_tbl_h_piece2(x)
     tl <- top_left(x) %||% rep("", length(h_rows))
     main_nls <- vapply(
       seq_along(h_rows),
-      function(i) max(nlines(h_rows[[i]], colwidths = colwidths), nlines(tl[i], colwidths = colwidths[1])),
+      function(i) {
+        max(
+          nlines(h_rows[[i]],
+            colwidths = colwidths,
+            fontspec = fontspec,
+            col_gap = col_gap
+          ),
+          nlines(tl[i],
+            colwidths = colwidths[1],
+            fontspec = fontspec
+          )
+        )
+      },
       1L
     )
 
@@ -106,7 +124,8 @@ setMethod(
         vapply(unlist(coldf$col_fnotes),
           nlines,
           1,
-          max_width = max_width
+          max_width = max_width,
+          fontspec = fontspec
         ),
         2 * divider_height(x)
       )
@@ -124,7 +143,12 @@ col_dfrow <- function(col,
                       nsibs = NA_integer_,
                       leaf_indices = cnum,
                       span = length(leaf_indices),
-                      col_fnotes = list()) {
+                      col_fnotes = list(),
+                      col_count = facet_colcount(col, NULL),
+                      ccount_visible = disp_ccounts(col),
+                      ccount_format = colcount_format(col),
+                      ccount_na_str,
+                      global_cc_format) {
   if (is.null(pth)) {
     pth <- pos_to_path(tree_pos(col))
   }
@@ -139,7 +163,11 @@ col_dfrow <- function(col,
     leaf_indices = I(list(leaf_indices)),
     total_span = span,
     col_fnotes = I(list(col_fnotes)),
-    n_col_fnotes = length(col_fnotes)
+    n_col_fnotes = length(col_fnotes),
+    col_count = col_count,
+    ccount_visible = ccount_visible,
+    ccount_format = ccount_format %||% global_cc_format,
+    ccount_na_str = ccount_na_str
   )
 }
 
@@ -149,11 +177,13 @@ pos_to_path <- function(pos) {
 
   path <- character()
   for (i in seq_along(spls)) {
+    nm <- obj_name(spls[[i]])
+    val_i <- value_names(vals[[i]])
     path <- c(
       path,
       obj_name(spls[[i]]),
       ## rawvalues(vals[[i]]))
-      value_names(vals[[i]])
+      if (!is.na(val_i)) val_i
     )
   }
   path
@@ -187,7 +217,9 @@ setMethod(
            repr_inds = integer(),
            sibpos = NA_integer_,
            nsibs = NA_integer_,
-           max_width = NULL) {
+           max_width = NULL,
+           fontspec = NULL,
+           col_gap = 3) {
     indent <- indent + indent_mod(tt)
     ## retained for debugging info
     orig_rownum <- rownum # nolint
@@ -217,7 +249,8 @@ setMethod(
           nsibs = nsibs,
           nrowrefs = 0L,
           ncellrefs = 0L,
-          nreflines = 0L
+          nreflines = 0L,
+          fontspec = fontspec
         ))
       )
     }
@@ -232,7 +265,8 @@ setMethod(
         incontent = TRUE,
         repr_ext = repr_ext,
         repr_inds = repr_inds,
-        max_width = max_width
+        max_width = max_width,
+        fontspec = fontspec
       )
       rownum <- max(newdf$abs_rownumber, na.rm = TRUE)
 
@@ -258,7 +292,8 @@ setMethod(
         incontent = TRUE,
         repr_ext = repr_ext,
         repr_inds = repr_inds,
-        max_width = max_width
+        max_width = max_width,
+        fontspec = fontspec
       )
       crnums <- contdf$abs_rownumber
       crnums <- crnums[!is.na(crnums)]
@@ -288,7 +323,8 @@ setMethod(
         repr_inds = repr_inds,
         nsibs = newnsibs,
         sibpos = i,
-        max_width = max_width
+        max_width = max_width,
+        fontspec = fontspec
       )
 
       #       print(kiddfs$abs_rownumber)
@@ -322,12 +358,23 @@ setMethod(
            repr_inds = integer(),
            sibpos = NA_integer_,
            nsibs = NA_integer_,
-           max_width = NULL) {
+           max_width = NULL,
+           fontspec,
+           col_gap = 3) {
     indent <- indent + indent_mod(tt)
     rownum <- rownum + 1
     rrefs <- row_footnotes(tt)
     crefs <- cell_footnotes(tt)
-    reflines <- sum(sapply(c(rrefs, crefs), nlines, colwidths = colwidths, max_width = max_width))
+    reflines <- sum(
+      sapply(
+        c(rrefs, crefs),
+        nlines,
+        colwidths = colwidths,
+        max_width = max_width,
+        fontspec = fontspec,
+        col_gap = col_gap
+      )
+    ) ## col_gap not strictly necessary as these aren't rows, but why not
     ret <- pagdfrow(
       row = tt,
       rnum = rownum,
@@ -338,12 +385,13 @@ setMethod(
       repext = repr_ext,
       repind = repr_inds,
       indent = indent,
-      extent = nlines(tt, colwidths = colwidths, max_width = max_width),
+      extent = nlines(tt, colwidths = colwidths, max_width = max_width, fontspec = fontspec, col_gap = col_gap),
       ## these two are unlist calls cause they come in lists even with no footnotes
       nrowrefs = length(rrefs),
       ncellrefs = length(unlist(crefs)),
       nreflines = reflines,
-      trailing_sep = trailing_section_div(tt)
+      trailing_sep = trailing_section_div(tt),
+      fontspec = fontspec
     )
     ret
   }
@@ -363,11 +411,13 @@ setMethod(
            repr_inds = integer(),
            sibpos = NA_integer_,
            nsibs = NA_integer_,
-           max_width = NULL) {
+           max_width = NULL,
+           fontspec,
+           col_gap = 3) {
     rownum <- rownum + 1
     indent <- indent + indent_mod(tt)
     ret <- pagdfrow(tt,
-      extent = nlines(tt, colwidths = colwidths, max_width = max_width),
+      extent = nlines(tt, colwidths = colwidths, max_width = max_width, fontspec = fontspec, col_gap = col_gap),
       rnum = rownum,
       colwidths = colwidths,
       sibpos = sibpos,
@@ -380,9 +430,12 @@ setMethod(
       ncellrefs = 0L,
       nreflines = sum(vapply(row_footnotes(tt), nlines, NA_integer_,
         colwidths = colwidths,
-        max_width = max_width
+        max_width = max_width,
+        fontspec = fontspec,
+        col_gap = col_gap
       )),
-      trailing_sep = trailing_section_div(tt)
+      trailing_sep = trailing_section_div(tt),
+      fontspec = fontspec
     )
     if (!labelrow_visible(tt)) {
       ret <- ret[0, , drop = FALSE]
@@ -397,7 +450,9 @@ setGeneric("inner_col_df", function(ct,
                                     colnum = 0L,
                                     sibpos = NA_integer_,
                                     nsibs = NA_integer_,
-                                    ncolref = 0L) {
+                                    ncolref = 0L,
+                                    na_str,
+                                    global_cc_format) {
   standardGeneric("inner_col_df")
 })
 
@@ -407,19 +462,25 @@ setGeneric("inner_col_df", function(ct,
 #' `data.frame`.
 #'
 #' @inheritParams formatters::make_row_df
-#'
+#' @param ccount_format (`FormatSpec`)\cr The format to be used by default for
+#'   column counts if one is not specified for an individual column count.
+#' @param na_str (`character(1)`)\cr The string to display when a column count is NA. Users should not need to set this.
 #' @export
 make_col_df <- function(tt,
                         colwidths = NULL,
-                        visible_only = TRUE) {
-  ctree <- coltree(tt) ## this is a null op if its already a coltree object
+                        visible_only = TRUE,
+                        na_str = "",
+                        ccount_format = colcount_format(tt) %||% "(N=xx)") {
+  ctree <- coltree(tt, ccount_format = colcount_format(tt)) ## this is a null op if its already a coltree object
   rows <- inner_col_df(ctree,
     ## colwidths is currently unused anyway...  propose_column_widths(matrix_form(tt, indent_rownames=TRUE)),
     colwidths = colwidths,
     visible_only = visible_only,
     colnum = 1L,
     sibpos = 1L,
-    nsibs = 1L
+    nsibs = 1L,
+    na_str = na_str,
+    global_cc_format = ccount_format
   ) ## nsiblings includes current so 1 means "only child"
 
   do.call(rbind, rows)
@@ -430,14 +491,18 @@ setMethod(
   function(ct, colwidths, visible_only,
            colnum,
            sibpos,
-           nsibs) {
+           nsibs,
+           na_str,
+           global_cc_format) {
     list(col_dfrow(
       col = ct,
       cnum = colnum,
       sibpos = sibpos,
       nsibs = nsibs,
       leaf_indices = colnum,
-      col_fnotes = col_footnotes(ct)
+      col_fnotes = col_footnotes(ct),
+      ccount_na_str = na_str,
+      global_cc_format = global_cc_format
     ))
   }
 )
@@ -447,7 +512,9 @@ setMethod(
   function(ct, colwidths, visible_only,
            colnum,
            sibpos,
-           nsibs) {
+           nsibs,
+           na_str,
+           global_cc_format) {
     kids <- tree_children(ct)
     ret <- vector("list", length(kids))
     for (i in seq_along(kids)) {
@@ -458,7 +525,9 @@ setMethod(
           colnum = colnum,
           sibpos = i,
           nsibs = length(kids),
-          visible_only = visible_only
+          visible_only = visible_only,
+          na_str = na_str,
+          global_cc_format = global_cc_format
         )
       )
       colnum <- max(newrows$abs_pos, colnum, na.rm = TRUE) + 1
@@ -476,7 +545,9 @@ setMethod(
           sibpos = sibpos,
           nsibs = nsibs,
           pth = thispth,
-          col_fnotes = col_footnotes(ct)
+          col_fnotes = col_footnotes(ct),
+          ccount_na_str = na_str,
+          global_cc_format = global_cc_format
         ))
         ret <- c(thisone, ret)
       }
@@ -488,13 +559,14 @@ setMethod(
 
 ## THIS INCLUDES BOTH "table stub" (i.e. column label and top_left) AND
 ## title/subtitle!!!!!
-.header_rep_nlines <- function(tt, colwidths, max_width, verbose = FALSE) {
-  cinfo_lines <- nlines(col_info(tt), colwidths = colwidths, max_width = max_width)
+.header_rep_nlines <- function(tt, colwidths, max_width, fontspec, verbose = FALSE) {
+  cinfo_lines <- nlines(col_info(tt), colwidths = colwidths, max_width = max_width, fontspec = fontspec)
   if (any(nzchar(all_titles(tt)))) {
     ## +1 is for blank line between subtitles and divider
     tlines <- sum(nlines(all_titles(tt),
       colwidths = colwidths,
-      max_width = max_width
+      max_width = max_width,
+      fontspec = fontspec
     )) + divider_height(tt) + 1L
   } else {
     tlines <- 0
@@ -512,12 +584,13 @@ setMethod(
 ## this is ***only*** lines that are expected to be repeated on  multiple pages:
 ## main footer, prov footer, and referential footnotes on **columns**
 
-.footer_rep_nlines <- function(tt, colwidths, max_width, have_cfnotes, verbose = FALSE) {
+.footer_rep_nlines <- function(tt, colwidths, max_width, have_cfnotes, fontspec, verbose = FALSE) {
   flines <- nlines(main_footer(tt),
     colwidths = colwidths,
-    max_width = max_width - table_inset(tt)
+    max_width = max_width - table_inset(tt),
+    fontspec = fontspec
   ) +
-    nlines(prov_footer(tt), colwidths = colwidths, max_width = max_width)
+    nlines(prov_footer(tt), colwidths = colwidths, max_width = max_width, fontspec = fontspec)
   if (flines > 0) {
     dl_contrib <- if (have_cfnotes) 0 else divider_height(tt)
     flines <- flines + dl_contrib + 1L
@@ -635,6 +708,8 @@ pag_tt_indices <- function(tt,
                            nosplitin = character(),
                            colwidths = NULL,
                            max_width = NULL,
+                           fontspec = NULL,
+                           col_gap = 3,
                            verbose = FALSE) {
   dheight <- divider_height(tt)
 
@@ -644,7 +719,8 @@ pag_tt_indices <- function(tt,
 
   hlines <- .header_rep_nlines(tt,
     colwidths = colwidths, max_width = max_width,
-    verbose = verbose
+    verbose = verbose,
+    fontspec = fontspec
   )
   ## if(any(nzchar(all_titles(tt)))) {
   ##     tlines <- sum(nlines(all_titles(tt), colwidths = colwidths, max_width = max_width)) +
@@ -661,8 +737,11 @@ pag_tt_indices <- function(tt,
   ##     flines <- flines + dl_contrib + 1L
   ## }
   flines <- .footer_rep_nlines(tt,
-    colwidths = colwidths, max_width = max_width,
-    have_cfnotes = have_cfnotes, verbose = verbose
+    colwidths = colwidths,
+    max_width = max_width,
+    have_cfnotes = have_cfnotes,
+    fontspec = fontspec,
+    verbose = verbose
   )
   ## row lines per page
   rlpp <- lpp - hlines - flines
@@ -679,7 +758,9 @@ pag_tt_indices <- function(tt,
     nosplitin = nosplitin,
     verbose = verbose,
     have_col_fnotes = have_cfnotes,
-    div_height = dheight
+    div_height = dheight,
+    col_gap = col_gap,
+    has_rowlabels = TRUE
   )
 }
 
@@ -773,7 +854,14 @@ paginate_table <- function(tt,
                            colwidths = NULL,
                            tf_wrap = FALSE,
                            max_width = NULL,
+                           fontspec = font_spec(font_family, font_size, lineheight),
+                           col_gap = 3,
                            verbose = FALSE) {
+  new_dev <- open_font_dev(fontspec)
+  if (new_dev) {
+    on.exit(close_font_dev())
+  }
+
   if ((non_null_na(lpp) || non_null_na(cpp)) &&
     (!is.null(page_type) || (!is.null(pg_width) && !is.null(pg_height)))) { # nolint
     pg_lcpp <- page_lcpp(
@@ -784,7 +872,8 @@ paginate_table <- function(tt,
       pg_width = pg_width,
       pg_height = pg_height,
       margins = margins,
-      landscape = landscape
+      landscape = landscape,
+      fontspec = fontspec
     )
 
     if (non_null_na(lpp)) {
@@ -803,7 +892,15 @@ paginate_table <- function(tt,
   }
 
   if (is.null(colwidths)) {
-    colwidths <- propose_column_widths(matrix_form(tt, indent_rownames = TRUE))
+    colwidths <- propose_column_widths(
+      matrix_form(
+        tt,
+        indent_rownames = TRUE,
+        fontspec = fontspec,
+        col_gap = col_gap
+      ),
+      fontspec = fontspec
+    )
   }
 
   if (!tf_wrap) {
@@ -815,7 +912,7 @@ paginate_table <- function(tt,
     max_width <- cpp
   } else if (identical(max_width, "auto")) {
     ## XXX this 3 is column sep width!!!!!!!
-    max_width <- sum(colwidths) + 3 * (length(colwidths) - 1)
+    max_width <- sum(colwidths) + col_gap * (length(colwidths) - 1)
   }
   if (!is.null(cpp) && !is.null(max_width) && max_width > cpp) {
     warning("max_width specified is wider than characters per page width (cpp).")
@@ -837,16 +934,19 @@ paginate_table <- function(tt,
       colwidths = colwidths,
       tf_wrap = tf_wrap,
       max_width = max_width,
-      verbose = verbose
+      fontspec = fontspec,
+      verbose = verbose,
+      col_gap = col_gap
     )
     return(unlist(ret, recursive = TRUE))
   }
 
   inds <- paginate_indices(tt,
     page_type = page_type,
-    font_family = font_family,
-    font_size = font_size,
-    lineheight = lineheight,
+    fontspec = fontspec,
+    ## font_family = font_family,
+    ## font_size = font_size,
+    ## lineheight = lineheight,
     landscape = landscape,
     pg_width = pg_width,
     pg_height = pg_height,
@@ -858,6 +958,7 @@ paginate_table <- function(tt,
     colwidths = colwidths,
     tf_wrap = tf_wrap,
     max_width = max_width,
+    col_gap = col_gap,
     verbose = verbose
   ) ## paginate_table apparently doesn't accept indent_size
 
