@@ -1,7 +1,20 @@
-## ---- echo=FALSE--------------------------------------------------------------
+## ----include = FALSE----------------------------------------------------------
+suggested_dependent_pkgs <- c("dplyr")
+knitr::opts_chunk$set(
+  collapse = TRUE,
+  comment = "#>",
+  eval = all(vapply(
+    suggested_dependent_pkgs,
+    requireNamespace,
+    logical(1),
+    quietly = TRUE
+  ))
+)
+
+## ----echo=FALSE---------------------------------------------------------------
 knitr::opts_chunk$set(comment = "#")
 
-## ---- message=FALSE-----------------------------------------------------------
+## ----message=FALSE------------------------------------------------------------
 library(rtables)
 library(dplyr)
 
@@ -16,7 +29,7 @@ qtable(ex_adsl, col_vars = "ARM", row_vars = "SEX")
 ## -----------------------------------------------------------------------------
 qtable(ex_adsl, "ARM", show_colcounts = FALSE)
 
-## ---- eval = FALSE------------------------------------------------------------
+## ----eval = FALSE-------------------------------------------------------------
 #  tmp_adsl <- ex_adsl
 #  tmp_adsl$new <- rep_len(c("", "A", "B"), nrow(tmp_adsl))
 #  
