@@ -225,12 +225,6 @@ test_that("header sep setting works", {
 })
 
 # section_div tests ------------------------------------------------------------
-check_pattern <- function(element, letter, len) {
-  # Regular expression to match exactly len of the same letter
-  regex <- paste0(rep(letter, len), collapse = "")
-  return(grepl(regex, element, fixed = TRUE))
-}
-
 test_structure_with_a_getter <- function(tbl, getter, val_per_lev) {
   # Main table obj
   expect_identical(tbl %>% getter(), val_per_lev$global)
@@ -252,11 +246,11 @@ test_structure_with_a_getter <- function(tbl, getter, val_per_lev) {
 
   # The elementary table has it?
   leaves_elementary_tbl <- tree_children(split1)[[1]]
-  expect_identical(leaves_elementary_tbl %>% getter(), val_per_lev$elem_tbl)
+  expect_identical(leaves_elementary_tbl %>% getter(), val_per_lev$elem_tbl_labelrow)
   expect_identical(tt_labelrow(leaves_elementary_tbl) %>% getter(), val_per_lev$elem_tbl_labelrow)
 
   # Data rows has it?
-  for (i in 1:nrow(leaves_elementary_tbl)) {
+  for (i in seq_len(nrow(leaves_elementary_tbl))) {
     expect_identical(tree_children(leaves_elementary_tbl)[[i]] %>% getter(), val_per_lev$datarow[i])
   }
 }
@@ -405,3 +399,20 @@ test_that("header_section_div works", {
 
   expect_true(check_pattern(header_sdiv, "+", nchar(header_sdiv)))
 })
+
+test_that("top_level_section_div works", {
+  lyt <- basic_table(top_level_section_div = "a") %>%
+    split_cols_by("ARM") %>%
+    split_rows_by("SEX", split_fun = drop_split_levels) %>%
+    analyze("AGE") %>%
+    split_rows_by("RACE", split_fun = drop_split_levels) %>%
+    split_rows_by("SEX", split_fun = drop_split_levels) %>%
+    analyze("AGE")
+  tbl <- build_table(lyt, DM)
+  expect_identical(top_level_section_div(lyt), "a")
+  top_level_section_div(lyt) <- "="
+  expect_identical(top_level_section_div(lyt), "=")
+  tbl <- build_table(lyt, DM)
+  top_lev_div_str <- strsplit(toString(tbl), "\n")[[1]][7]
+  expect_true(check_pattern(top_lev_div_str, "=", nchar(top_lev_div_str)))
+})
