@@ -340,6 +340,7 @@ test_that("Various Printing things work", {
 
   table_structure(tab, detail = "subtable") ## treestruct(tab)
   table_structure(tab, detail = "row") ## treestruct(tab)
+  coltree_structure(tab)
 
   ## this is not intended to be a valid layout, it just
   ## tries to hit every type of split for the print machinery
@@ -554,6 +555,9 @@ test_that("Cell and column label wrapping works in printing", {
 
 
 test_that("row label indentation is kept even if there are newline characters", {
+  skip_if_not_installed("dplyr")
+  require(dplyr, quietly = TRUE)
+
   ANL <- DM %>%
     mutate(value = rnorm(n()), pctdiff = runif(n())) %>%
     filter(ARM == "A: Drug X")
@@ -785,3 +789,145 @@ test_that("horizontal separator is propagated from table to print and export", {
   export_txt_tbl <- strsplit(export_as_txt(tbl), "\n")[[1]]
   expect_identical(tostring_tbl, export_txt_tbl)
 })
+
+## higher-level showing ncols works:
+
+test_that("showing higher-level ncols works", {
+  mydat <- subset(ex_adsl, SEX %in% c("M", "F"))
+  mydat$SEX2 <- factor(
+    ifelse(
+      mydat$SEX == "M",
+      "males",
+      "super long sentence that involves females"
+    )
+  )
+
+  lyt <- basic_table() %>%
+    split_cols_by("ARM", show_colcounts = TRUE) %>%
+    split_cols_by("SEX2", show_colcounts = TRUE) %>%
+    split_cols_by("STRATA1") %>%
+    analyze("AGE")
+
+  tbl <- build_table(lyt, mydat)
+  expect_equal(colcount_na_str(tbl), "")
+  colcount_na_str(tbl) <- "wut"
+  expect_equal(colcount_na_str(tbl), "wut")
+  colcount_na_str(tbl) <- ""
+  cwds <- rep(8, ncol(tbl) + 1)
+  expect_equal(nlines(col_info(tbl), colwidths = cwds, fontspec = NULL), 7)
+  mpf <- matrix_form(tbl, TRUE)
+  ## this is to get around complaints about ::: in the precommit rules
+  dcfnw <- get("do_cell_fnotes_wrap", asNamespace("formatters"))
+  mpf <- dcfnw(mpf, cwds, NULL, FALSE, fontspec = NULL)
+  strs <- mf_strings(mpf)
+  ## wrapping some cells and not others still works
+  expect_equal(strs[3:4, 2], c("", "males"))
+
+  expect_equal(strs[2, 2], "(N=130)")
+  ## N= cells all across rows 2 (for ARM) and 5 (for SEX2), except rowlabels
+  expect_true(all(grepl("(N=", strs[c(2, 5), -1], fixed = TRUE)))
+  ## No N= cells elsewhere
+  expect_true(all(!grepl("(N=", strs[-c(2, 5), -1], fixed = TRUE)))
+
+  broken_tbl <- tbl
+  expect_true(colcount_visible(broken_tbl, c("ARM", "A: Drug X", "SEX2", "males")))
+  colcount_visible(broken_tbl, c("ARM", "A: Drug X", "SEX2", "males")) <- FALSE
+  expect_error(print(broken_tbl), "different colcount visibility among sibling facets")
+
+  ## does the old accessor still work ok
+
+  lyt2 <- basic_table() %>%
+    split_cols_by("ARM", show_colcounts = TRUE) %>%
+    split_cols_by("SEX2", show_colcounts = TRUE) %>%
+    split_cols_by("STRATA1", show_colcounts = TRUE) %>%
+    analyze("AGE")
+
+  tbl2 <- build_table(lyt2, mydat)
+  nc <- ncol(tbl2)
+  new_ccs <- seq_len(nc)
+
+  col_counts(tbl2) <- new_ccs
+
+  mpf2 <- matrix_form(tbl2, TRUE)
+  expect_equal(
+    mf_strings(mpf2)[mf_nlheader(mpf2), -1, drop = TRUE],
+    sprintf("(N=%d)", new_ccs)
+  )
+  ## NA counts (to display blank) work correctly for higher level facets
+
+  tbl3 <- tbl
+  facet_colcount(tbl3, c("ARM", "C: Combination")) <- NA_integer_
+  mpf3 <- matrix_form(tbl3, TRUE)
+  ## starting at "column" 2 because topleft/row labels
+  expect_equal(
+    mf_strings(mpf3)[2, 2:13],
+    mf_strings(mpf)[2, 2:13]
+  )
+  expect_equal(
+    mf_strings(mpf3)[2, 14:19],
+    rep("", 6)
+  )
+
+  tbl4 <- tbl2
+  col_counts(tbl4)[rep(c(FALSE, TRUE), times = c(14, 4))] <- NA_integer_
+
+  adsl <- ex_adsl
+
+  adsl$active_trt <- factor(ifelse(grepl("Placebo", adsl$ARM), " ", "Active Treatment Group"))
+  adsl$rr_header <- "Risk Difference % CI"
+
+  combodf <- tribble(
+    ~valname, ~label, ~levelcombo, ~exargs,
+    "A_C", "Arms A+C", c("A: Drug X", "C: Combination"), list()
+  )
+
+  lyt5 <- basic_table(show_colcounts = TRUE) %>%
+    split_cols_by("active_trt", split_fun = trim_levels_in_group("ARM")) %>%
+    split_cols_by("ARM", split_fun = add_combo_levels(combodf)) %>%
+    split_cols_by("rr_header", nested = FALSE) %>%
+    split_cols_by("ARM", split_fun = keep_split_levels(c("A: Drug X", "C: Combination"))) %>%
+    analyze("AGE")
+
+  tbl5 <- build_table(lyt5, adsl)
+  expect_silent(toString(tbl5))
+  col_counts(tbl5)[c(FALSE, TRUE, FALSE, FALSE, FALSE, TRUE, TRUE)] <- NA_integer_
+  mpf5 <- matrix_form(tbl5, TRUE)
+  expect_equal(
+    mf_strings(mpf5)[3, c(3, 7, 8)], # cols 2, 6 and 7, remember row labels!
+    c("", "", "")
+  )
+
+  ## turning counts for a facet's children off is different than setting
+  ## the visible counts to NA, note alignment here, no spaces under risk diff
+  ## arms
+  facet_colcounts_visible(tbl5, c("rr_header", "Risk Difference % CI", "ARM")) <- FALSE
+  mpf5b <- matrix_form(tbl5, TRUE)
+  expect_equal(
+    mf_strings(mpf5b)[3, 7:8],
+    c("A: Drug X", "C: Combination")
+  )
+  lyt6 <- basic_table(show_colcounts = TRUE, colcount_format = "N=xx") %>%
+    split_cols_by("active_trt", split_fun = trim_levels_in_group("ARM")) %>%
+    split_cols_by("ARM", split_fun = add_combo_levels(combodf), show_colcounts = TRUE, colcount_format = "(N=xx)") %>%
+    split_cols_by("rr_header", nested = FALSE) %>%
+    split_cols_by("ARM", split_fun = keep_split_levels(c("A: Drug X", "C: Combination"))) %>%
+    analyze("AGE")
+
+  tbl6 <- build_table(lyt6, adsl)
+
+  lyt7 <- basic_table(show_colcounts = TRUE, colcount_format = "N=xx") %>%
+    split_cols_by("active_trt", split_fun = trim_levels_in_group("ARM")) %>%
+    split_cols_by("ARM", split_fun = add_combo_levels(combodf), show_colcounts = TRUE, colcount_format = "(N=xx)") %>%
+    split_cols_by("STRATA1") %>%
+    split_cols_by("rr_header", nested = FALSE) %>%
+    split_cols_by("ARM", split_fun = keep_split_levels(c("A: Drug X", "C: Combination"))) %>%
+    analyze("AGE")
+
+  tbl7 <- build_table(lyt7, adsl)
+  expect_silent(toString(tbl7))
+
+  mpf7 <- matrix_form(tbl7)
+  strs7 <- mf_strings(mpf7)
+  expect_equal(length(grep("^[(]N=", strs7)), 15) ## cause of spanning, 5 visible counts, each span 3
+  expect_equal(length(grep("^N=", strs7)), ncol(tbl7))
+})
