@@ -1,6 +1,9 @@
 context("Analysis functions (afun)")
 
 test_that(".spl_context contains information about the column split", {
+  skip_if_not_installed("dplyr")
+  require(dplyr, quietly = TRUE)
+
   ## Duplication hack -> This would need to use  split_cols_by_multivar(...)
   # Workaround for #690
   DM_tmp <- DM %>%
@@ -39,6 +42,12 @@ test_that(".spl_context contains information about the column split", {
 })
 
 test_that(".spl_context and afun extra parameters contain information about combo counts", {
+  skip_if_not_installed("tibble")
+  require(tibble, quietly = TRUE)
+
+  skip_if_not_installed("dplyr")
+  require(dplyr, quietly = TRUE)
+
   ## Fix for https://github.com/insightsengineering/rtables/issues/517
   combodf <- tribble(
     ~valname, ~label, ~levelcombo, ~exargs,
@@ -217,6 +226,9 @@ test_that("Error localization for missing split variable when done in alt_count_
 })
 
 test_that("Error localization for missmatch split variable when done in alt_count_df", {
+  skip_if_not_installed("dplyr")
+  require(dplyr, quietly = TRUE)
+
   afun_tmp <- function(x, .alt_df_row, .spl_context, ...) {
     # Important check that order is aligned even if source levels are not
     check_val <- unique(.alt_df_row$ARMCD)
@@ -281,6 +293,9 @@ test_that("Error localization for missmatch split variable when done in alt_coun
 context("Content functions (cfun)")
 
 test_that(".alt_df_row appears in cfun but not in afun.", {
+  skip_if_not_installed("dplyr")
+  require(dplyr, quietly = TRUE)
+
   # Adding STRATA2 col to DM for alt_counts_df col split
   alt_tmp <- DM %>% left_join(
     ex_adsl %>%
